# Concept: raw pointers

A raw pointer in C++ is a basic pointer type that holds the memory address of another variable or object. Raw pointers are fundamental to C++ and provide direct access to memory, but they come with significant responsibilities and potential risks. Unlike smart pointers (such as `std::unique_ptr` or `std::shared_ptr)`, raw pointers do not manage the lifetime of the object they point to, meaning that the programmer is responsible for ensuring proper memory management.

## Key Characteristics of Raw Pointers

- Direct Memory Access: A raw pointer stores the address of a variable or object, allowing direct access to that memory.
- No Automatic Resource Management: Raw pointers do not automatically free the memory they point to. If memory is dynamically allocated, the programmer must manually deallocate it using `delete` or `delete[].
- Nullability: A raw pointer can be a null pointer, meaning it points to no object or memory. The null pointer is typically represented by the nullptr keyword in modern C++.
- Pointer Arithmetic: Raw pointers support arithmetic operations, such as incrementing to point to the next memory location or subtracting to find the difference between addresses.

## Example 1: Simple Raw Pointer

```cpp
#include <iostream>

int main() {
    int x = 42;
    int* ptr = &x;  // 'ptr' is a raw pointer to 'x'

    std::cout << "Value of x: " << x << std::endl;
    std::cout << "Value pointed to by ptr: " << *ptr << std::endl;

    *ptr = 100;  // Modify the value of 'x' through the pointer
    std::cout << "New value of x: " << x << std::endl;

    return 0;
}
```

### Explanation:

- `int* ptr = &x;`: `ptr` is a raw pointer that holds the address of `x`.
- Dereferencing (`*ptr`): Dereferencing the pointer ptr gives access to the value stored at the memory address it points to, which is `x`.
- Modifying through Pointer: The value of `x` can be modified by dereferencing `ptr`.

## Example 2: Dynamic Memory Allocation with Raw Pointers

Raw pointers are commonly used for dynamic memory allocation with new and delete.
```cpp
#include <iostream>

int main() {
    int* ptr = new int(42);  // Dynamically allocate an integer and initialize it to 42

    std::cout << "Value pointed to by ptr: " << *ptr << std::endl;

    delete ptr;  // Free the dynamically allocated memory

    // ptr now becomes a dangling pointer; it points to a location that has been freed.
    ptr = nullptr;  // It's good practice to set ptr to nullptr after deletion

    return 0;
}
```

### Explanation:

- Dynamic Allocation: `int* ptr = new int(42);` dynamically allocates memory for an integer and initializes it to `42`. `ptr` holds the address of this memory.
- Manual Deallocation: `delete ptr;` deallocates the memory pointed to by `ptr`. Failing to deallocate this memory can result in a memory leak.
- Nullifying the Pointer: After deleting the memory, setting `ptr` to `nullptr` is a good practice to avoid accidentally accessing a dangling pointer.

## Example 3: Using a Null Pointer

A null pointer is a pointer that does not point to any object or memory location. In modern C++, the nullptr keyword is used to represent a null pointer.

```cpp
#include <iostream>

int main() {
    int* ptr = nullptr;  // 'ptr' is a null pointer

    if (ptr == nullptr) {
        std::cout << "ptr is null." << std::endl;
    }

    return 0;
}
```

### Explanation:

- `nullptr`: `ptr` is initialized to `nullptr`, indicating that it does not point to any valid memory.
- Null Check: Before dereferencing a pointer, it's common practice to check if it is null to avoid undefined behavior.

## Example 4: Pointer Arithmetic with Arrays

Raw pointers support pointer arithmetic, which is often used in array manipulation and traversal.

```cpp
#include <iostream>

int main() {
    int arr[5] = {10, 20, 30, 40, 50};
    int* ptr = arr;  // 'ptr' points to the first element of the array

    for (int i = 0; i < 5; ++i) {
        std::cout << "Value at index " << i << ": " << *(ptr + i) << std::endl;
    }

    return 0;
}
```

### Explanation:

- Pointer to Array: `int* ptr = arr`; points to the first element of the array `arr`.
- Pointer Arithmetic: `ptr + i` moves the pointer to the i-th element in the array. Dereferencing this pointer with `*(ptr + i)` gives the value at that index.

## Example 5: Dangling Pointer

A dangling pointer is a pointer that refers to memory that has been freed or is otherwise invalid. Dereferencing a dangling pointer can lead to undefined behavior.

```cpp
#include <iostream>

int main() {
    int* ptr = new int(42);  // Dynamically allocate memory

    delete ptr;  // Free the memory

    // ptr is now a dangling pointer
    // std::cout << *ptr << std::endl;  // Dereferencing ptr here would be unsafe

    ptr = nullptr;  // Prevent dangling by setting ptr to nullptr

    return 0;
}
```

### Explanation:

- Dangling Pointer: After `delete ptr;`, the pointer ptr still holds the address of the freed memory, making it a dangling pointer.
- Preventing Dangling: Setting ptr to `nullptr` after deletion ensures that it no longer refers to invalid memory.

## Common Pitfalls with Raw Pointers

- Memory Leaks: Forgetting to delete memory allocated with new leads to memory leaks.
- Dangling Pointers: Accessing memory after it has been freed results in dangling pointers, leading to undefined behavior.
- Null Pointer Dereferencing: Dereferencing a null pointer results in a runtime error.

## Summary

- Raw Pointers: Basic pointers that store memory addresses and provide direct access to memory. They require manual management of the memory they point to.
- Dynamic Memory: Raw pointers are often used with new and delete for dynamic memory allocation, but this requires careful management to avoid memory leaks and dangling pointers.
- Pointer Arithmetic: Raw pointers support arithmetic operations, allowing traversal and manipulation of arrays and memory blocks.
- Null Pointers: A null pointer (`nullptr`) represents a pointer that does not point to any valid memory.

Raw pointers are a fundamental feature of C++, offering powerful control over memory and performance, but they come with significant responsibilities and risks. Modern C++ often prefers smart pointers for managing dynamically allocated memory due to their automatic memory management features.
